/* global value list
USE_BONE	= false
USE_NORMALMAP	= TRUE
LIGHTING	= TRUE
NEED_VIEWSPACE_NORMAL	= false
NEED_NORMAL	= TRUE
USE_SUNSPEC	= TRUE
NEED_WORLD_POS	= TRUE
NEED_WORLD_NORMAL	= false
NEED_SPEC	= false
*/


	
#define	Float	highp float
#define	Float2	highp vec2
#define	Float3	highp vec3
#define	Float4	highp vec4
#define	Float3x3	highp mat3
#define	Float4x4	highp mat4
#define	Half	mediump float
#define	Half2	mediump vec2
#define	Half3	mediump vec3
#define	Half4	mediump vec4
#define	Fixed	lowp float
#define	Fixed2	lowp vec2
#define	Fixed3	lowp vec3
#define	Fixed4	lowp vec4
#define	saturate(x)	clamp(x,0.0,1.0)
#define	lerp(a,b,w)	mix(a,b,w)
#define	tex1D		texture1D
#define	tex2D		texture2D
#define	MAX_BONE_COUNT	40
#define	MAX_BATCH_COUNT	40

		
uniform Float4x4	u_tm_wvp;
			
uniform Float4x4	u_tm_world;
			
uniform Float4x4	u_tm_view;
		
uniform Float4x4	u_tm_inv_view;
		
// Lighting
			
uniform Float4	u_ambient;
			
#define	MAX_POINT_LIGHT_COUNT	4
uniform Float4	u_point_light_array[MAX_POINT_LIGHT_COUNT*2+1];
			
uniform Float4	u_m_light_pos;
					
attribute Float3	a_Pos;
		
attribute Half3	a_Nor;
		
attribute Half3	a_Tag;
attribute Half3	a_Binor;
		
attribute Fixed4	a_Diff;
		
attribute Half2	a_Tex0;
		
varying Half4	v_Color;
		
varying Half3	v_MainLightDir;
varying Half	v_Dist;
				
varying Half2	v_TexDiff;
			

void main()
{
		
        Float4 pos = vec4(a_Pos.xyz,1.0);
        
          Half3 nor = a_Nor;
        
          Half3 tag = a_Tag;
          Half3 binor = a_Binor;
        
	gl_Position = u_tm_wvp * pos;
				
	Float3 worldPos	= (u_tm_world * pos).xyz;
				
	Fixed4 vc = a_Diff;
			
	Fixed4 color = u_ambient * vc;
				
	Half3 N = normalize(mat3(u_tm_world)*nor);
			
	Half NdotL;
			
	int point_light_cnt = int(u_point_light_array[0].x)*2;
	if (point_light_cnt > 0)
	{
		for (int i=1; i <= point_light_cnt; i+=2)
		{
			Float4 light_pos = u_point_light_array[i];
			Float4 light_col = u_point_light_array[i+1];
			Half3 dir = light_pos.xyz - worldPos.xyz;
			Half dist = length(dir);
			dir /= dist;
			dist = min(dist,light_pos.w) / light_pos.w;
			//dist *= dist;
			dist *= dist;
			dist = 1.0 - dist;
			NdotL = max(0.0, dot(N,dir));
			color.rgb += light_col.rgb * NdotL * dist;
		}
	}
			
	Float3 wT = normalize(mat3(u_tm_world)*tag);
	Float3 wB = normalize(mat3(u_tm_world)*binor);
	Float3 wN = normalize(mat3(u_tm_world)*nor);
	Float3x3 sunTagSpaceXform = mat3(wT,wB,wN);
				
	Float3 dir = u_m_light_pos.xyz - worldPos.xyz;
	Float dist = length(dir);
	dir /= dist;
	dist /= u_m_light_pos.w;
	v_MainLightDir = dir*sunTagSpaceXform;
	v_Dist = dist;
					
	v_Color = color;
		
	v_TexDiff.xy = a_Tex0;
				
}
	